(self.webpackChunkjupyter_jsmol=self.webpackChunkjupyter_jsmol||[]).push([[568],{568:(e,t,s)=>{e.exports=s(367),e.exports.version=s(147).version},367:(e,t,s)=>{var o=s(337);s(431);class l extends o.DOMWidgetModel{defaults(){return{...super.defaults(),_model_name:"JsmolModel",_view_name:"JsmolView",_model_module:"jupyter-jsmol",_view_module:"jupyter-jsmol",_model_module_version:"2022.1.0",_view_module_version:"2022.1.0",_info:{},_loaded:!1}}}class i extends o.DOMWidgetView{initialize(e){super.initialize(e),this.model.on("msg:custom",this.on_custom_message,this),this.model.on("destroy",this.on_destroy,this)}update(e){super.update(e)}render(){let e="jsmol_"+this.cid,t=Object.assign({width:"100%",height:"100%",use:"HTML5",j2sPath},this.model.get("_info"));Jmol.setDocument(!1),this._applet=Jmol.getApplet(e,t),this.el.innerHTML=Jmol.getAppletHtml(this._applet),Jmol.coverApplet(this._applet),this.model.set("_loaded",!0),this.touch()}on_custom_message(e){if("call"==e.type)switch(e.func){case"fullscreen":this.fullscreen();break;case"script":this.script(e.data);break;case"evaluate":this.send({type:"response",func:"evaluate",data:this.evaluate(e.data)});break;case"property":this.send({type:"response",func:"property",data:this.property(e.data)});break;default:console.log("there is no method for "+e.func)}}script(e){Jmol.script(this._applet,e)}evaluate(e){return Jmol.evaluateVar(this._applet,e)}property(e){return Jmol.getPropertyAsArray(this._applet,e)}on_destroy(){delete window[this._applet._id],delete this._applet}}e.exports={JsmolModel:l,JsmolView:i}},147:e=>{"use strict";e.exports=JSON.parse('{"version":"2022.1.0"}')}}]);