define((()=>(()=>{var e={758:(e,t,n)=>{n.p=document.querySelector("body").getAttribute("data-base-url")+"nbextensions/jupyter-jsmol",window.j2sPath=n.p+"/jsmol/j2s",window.require&&window.require.config({map:{"*":{"jupyter-jsmol":"nbextensions/jupyter-jsmol/index"}}}),e.exports={load_ipython_extension:function(){const e=document.createElement("script");e.src=n.p+"/jsmol/JSmol.min.nojq.js",e.async=!1,document.querySelector("head").appendChild(e)}}}},t={};function n(o){var r=t[o];if(void 0!==r)return r.exports;var s=t[o]={exports:{}};return e[o](s,s.exports,n),s.exports}return n.p="",n(758)})()));